{%- from "macros/form.html" import m_field %}
<table class="table js-widget model-field-list" data-init-with="dynamicRowWidget" data-prefix="{{ field.name }}">
  <thead>
  <tr>
    <th></th>
  </tr>
  </thead>

  <tbody>
  {%- for fieldform in field %}
    <tr>
      <td>
        {%- for subfield in fieldform %}
          {%- set attrs={'data-short-name': subfield.short_name} %}
          {{ m_field(subfield, **attrs) }}
        {%- endfor %}
      </td>
    </tr>
  {%- endfor %}
  </tbody>
</table>
